@@ -13,7 +13,8 @@ var DimpCompose = {
     //   atc_context, auto_save_interval, compose_cursor, disabled,
     //   drafts_mbox, editor_wait, fwdattach, is_popup, knl, md5_hdrs,
     //   md5_msg, md5_msgOrig, onload_show, old_action, old_identity, rte,
-    //   rte_loaded, sc_submit, skip_spellcheck, spellcheck, tasks, uploading
+    //   rte_loaded, sc_submit, skip_spellcheck, spellcheck, tasks, uploading,
+    //   upload_limit
 
     checkbox_context: $H({
         ctx_atc: $H({
@@ -80,6 +81,7 @@ var DimpCompose = {
     closeQReply: function()
     {
         this.md5_hdrs = this.md5_msg = this.md5_msgOrig = '';
+        this.upload_limit = false;
 
         $('attach_list').hide().childElements().each(this.removeAttachRow.bind(this));
         this.getCacheElt().clear();
@@ -294,9 +296,7 @@ var DimpCompose = {
                 return this.closeCompose();
 
             case 'addAttachment':
-                this.uploading = false;
-                $('upload_wait').hide();
-                this.initAttachList();
+                this._addAttachmentEnd();
                 break;
             }
         } else {
@@ -586,27 +586,44 @@ var DimpCompose = {
         // Set auto-save-drafts now if not already active.
         if (DimpCore.conf.auto_save_interval_val &&
             !this.auto_save_interval) {
-            this.auto_save_interval = new PeriodicalExecuter(function() {
-                if ($('compose').visible()) {
-                    var hdrs = murmurhash3($('to', 'cc', 'bcc', 'subject').compact().invoke('getValue').join('\0'), this.seed), msg;
-                    if (this.md5_hdrs) {
-                        msg = this.msgHash();
-                        if (this.md5_hdrs != hdrs || this.md5_msg != msg) {
-                            this.uniqueSubmit('autoSaveDraft');
-                        }
-                    } else {
-                        msg = this.md5_msgOrig;
-                    }
-                    this.md5_hdrs = hdrs;
-                    this.md5_msg = msg;
-                }
-            }.bind(this), DimpCore.conf.auto_save_interval_val * 60);
+            this.auto_save_interval = new PeriodicalExecuter(
+                this.autoSaveDraft.bind(this),
+                DimpCore.conf.auto_save_interval_val * 60
+            );
 
             /* Immediately execute to get hash of headers. */
             this.auto_save_interval.execute();
         }
     },
 
+    autoSaveDraft: function()
+    {
+        var hdrs, msg;
+
+        if (!$('compose').visible()) {
+            return;
+        }
+
+        hdrs = murmurhash3(
+            [$('to', 'cc', 'bcc', 'subject').compact().invoke('getValue'),
+             $('attach_list').select('SPAN.attachName').pluck('textContent')
+            ].flatten().join('\0'),
+            this.seed
+        );
+
+        if (this.md5_hdrs) {
+            msg = this.msgHash();
+            if (this.md5_hdrs != hdrs || this.md5_msg != msg) {
+                this.uniqueSubmit('autoSaveDraft');
+            }
+        } else {
+            msg = this.md5_msgOrig;
+        }
+
+        this.md5_hdrs = hdrs;
+        this.md5_msg = msg;
+    },
+
     msgHash: function()
     {
         return murmurhash3(ImpComposeBase.editor_on ? this.rte.getData() : $F('composeMessage'), this.seed);
@@ -667,9 +684,9 @@ var DimpCompose = {
 
     focusEditor: function()
     {
-        try {
+        if (this.rte.focus) {
             this.rte.focus();
-        } catch (e) {
+        } else {
             this.focusEditor.bind(this).defer();
         }
     },
@@ -762,6 +779,14 @@ var DimpCompose = {
 
         u.clear();
 
+        if (this.upload_limit) {
+            $('upload_limit').show();
+            u.up().hide();
+        } else {
+            $('upload_limit').hide();
+            u.up().show();
+        }
+
         if (!al.childElements().size()) {
             al.hide();
         }
@@ -769,6 +794,13 @@ var DimpCompose = {
         this.resizeMsgArea();
     },
 
+    _addAttachmentEnd: function()
+    {
+        this.uploading = false;
+        $('upload_wait').hide();
+        this.initAttachList();
+    },
+
     resizeMsgArea: function(e)
     {
         if (!document.loaded || $('dimpLoading').visible()) {
@@ -956,7 +988,7 @@ var DimpCompose = {
         case 'sendcc':
         case 'sendbcc':
         case 'sendto':
-            if (elt.match('TD.label SPAN')) {
+            if (DimpCore.conf.URI_ABOOK && elt.match('TD.label SPAN')) {
                 this.openAddressbook();
             }
             break;
@@ -1025,7 +1057,7 @@ var DimpCompose = {
                 tmp = elt.retrieve('popdown_id');
                 this.knl[tmp].knl.show();
                 this.knl[tmp].knl.ignoreClick(e.memo);
-                e.stop();
+                e.memo.stop();
             }
             break;
         }
@@ -1131,13 +1163,7 @@ var DimpCompose = {
 
         if (t['imp:compose']) {
             this.getCacheElt().setValue(t['imp:compose'].cacheid);
-            if (t['imp:compose'].atclimit) {
-                $('upload_limit').show();
-                $('upload').up().hide();
-            } else {
-                $('upload_limit').hide();
-                $('upload').up().show();
-            }
+            this.upload_limit = t['imp:compose'].atclimit;
         }
 
         if (t['imp:compose-atc']) {
@@ -1182,9 +1208,6 @@ var DimpCompose = {
             return;
         }
 
-        /* Resize height. */
-        window.resizeTo(window.outerWidth || document.documentElement.offsetWidth, screen.availHeight - (window.screenY || 0) - 50);
-
         /* Attach event handlers. */
         if (Prototype.Browser.IE) {
             // IE doesn't bubble change events.
@@ -1206,7 +1229,8 @@ var DimpCompose = {
                 }
             }.bindAsEventListener(this));
             DragHandler.dropelt = $('atcdrop');
-            DragHandler.droptarget = $('atctd');
+            DragHandler.droptarget = $('atcdiv');
+            DragHandler.hoverclass = 'atcdrop_over';
             DimpCore.addPopdown($('upload'), 'atc', {
                 no_offset: true
             });
@@ -1305,6 +1329,13 @@ document.observe('HordeCore:runTasks', function(e) {
     this.tasksHandler(e.memo);
 }.bindAsEventListener(DimpCompose));
 
+/* AJAX related events. */
+document.observe('HordeCore:ajaxFailure', function(e) {
+    if (this.uploading) {
+        this._addAttachmentEnd();
+    }
+}.bindAsEventListener(DimpCompose));
+
 
 /* Fix Ajax.Request#setRequestHeaders() behavior (Bug #12418).
  * (This is fixed in prototypejs as of December 2012.) */
