@@ -112,7 +112,9 @@ var ImpSearch = {
         }, this);
 
         if ($('search_criteria').childElements().size()) {
-            $('no_search_criteria', 'search_criteria').invoke('toggle');
+            if ($('no_search_criteria').visible()) {
+                $('no_search_criteria', 'search_criteria').invoke('toggle');
+            }
             this.showOr(true);
         }
     },
@@ -152,7 +154,7 @@ var ImpSearch = {
     resetCriteria: function()
     {
         var elts = $('search_criteria').childElements();
-        if (elts) {
+        if (elts.size()) {
             elts.invoke('remove');
             $('no_search_criteria', 'search_criteria').invoke('toggle');
             this.criteria = {};
@@ -310,7 +312,7 @@ var ImpSearch = {
     {
         var tmp = [
             new Element('EM').insert(this.text.flag),
-            new Element('SPAN', { className: 'searchFlag' }).insert(this.getCriteriaLabel(id).slice(0, -2)),
+            new Element('SPAN', { className: 'searchFlag' }).insert(this.getCriteriaLabel(id).slice(0, -2).escapeHTML()),
             new Element('SPAN', { className: 'notMatch' }).insert(new Element('INPUT', { className: 'checkbox', type: 'checkbox' }).setValue(not)).insert(this.text.not_match)
         ];
         this.criteria[this.insertCriteria(tmp)] = { t: id };
