@@ -224,7 +224,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
         $this->_base->queue->poll($mbox);
 
-        $vp = $this->_base->viewPortOb($mbox);
+        $vp = new IMP_Ajax_Application_Viewport($mbox);
         $vp->data_reset = 1;
         $vp->rowlist_reset = 1;
         $this->_base->addTask('viewport', $vp);
@@ -300,6 +300,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         } else {
             if ($initreload) {
                 $mask |= IMP_Imap_Tree::FLIST_ANCESTORS | IMP_Imap_Tree::FLIST_SAMELEVEL;
+
                 if ($GLOBALS['prefs']->getValue('nav_expanded')) {
                     $this->_base->queue->setMailboxOpt('expand', 1);
                     $mask |= IMP_Imap_Tree::FLIST_EXPANDED;
@@ -537,7 +538,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
             $this->_base->queue->poll(array_keys($this->_base->indices->indices()));
         }
 
-        $this->_base->addTask('viewport', $change ? $this->_base->viewPortData(true) : $this->_base->viewPortOb());
+        $this->_base->addTask('viewport', $change ? $this->_base->viewPortData(true) : new IMP_Ajax_Application_Viewport($this->_base->indices->mailbox));
 
         return true;
     }
@@ -701,6 +702,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
                     $notification->push(sprintf(_("Deleted attachment \"%s\"."), Horde_Mime::decode($imp_compose[$val]->getPart()->getName(true))), 'horde.success');
                     unset($imp_compose[$val]);
                     $result[] = $val;
+                    $this->_base->queue->compose($imp_compose);
                 }
             }
         }
@@ -899,7 +901,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
             if ($imp_compose->canUploadAttachment()) {
                 try {
-                    $atc_ob = $imp_compose->addAttachmentFromUpload($this->vars, 'upload');
+                    $atc_ob = $imp_compose->addAttachmentFromUpload('upload');
                     $atc_ob->related = true;
 
                     $data = array(
@@ -994,19 +996,26 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      *   - flagcolor: (string) Background color for flag label.
      *   - flagname: (string) Flag name.
      *
-     * @return boolean  True.
+     * @return object  An object with the following properties:
+     * <pre>
+     *   - success: (boolean) True if successful.
+     * </pre>
      */
     public function createFlag()
     {
         global $injector, $notification;
 
+        $ret = new stdClass;
+        $ret->success = true;
+
         $imp_flags = $injector->getInstance('IMP_Flags');
 
         try {
             $imp_flags->addFlag($this->vars->flagname);
         } catch (IMP_Exception $e) {
             $notification->push($e, 'horde.error');
-            return true;
+            $ret->success = false;
+            return $ret;
         }
 
         if (!empty($this->vars->flagcolor)) {
@@ -1021,10 +1030,10 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
         $name = 'imp:viewport';
         if ($this->_base->tasks->$name) {
-            IMP_Ajax_Application_ListMessages::addFlagMetadata($this->_base->tasks->$name, IMP_Mailbox::formFrom($this->_vars->mailbox));
+            $this->_base->tasks->$name->addFlagMetadata();
         }
 
-        return true;
+        return $ret;
     }
 
 }
