@@ -66,7 +66,7 @@ class IMP_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (6.1.4)';
+    public $version = 'H5 (6.1.6)';
 
     /**
      * Server key used in logged out session.
@@ -120,6 +120,10 @@ class IMP_Application extends Horde_Registry_Application
         if (empty($injector->getInstance('IMP_Imap')->config->admin)) {
             $this->auth = array_diff($this->auth, array('add', 'list', 'remove'));
         }
+
+        /* Set exception handler to handle uncaught
+         * Horde_Imap_Client_Exceptions. */
+        set_exception_handler(array($this, 'exceptionHandler'));
     }
 
     /**
@@ -509,10 +513,12 @@ class IMP_Application extends Horde_Registry_Application
         switch ($vars->actionID) {
         case 'download_all':
             $view_ob = new IMP_Contents_View(new IMP_Indices_Mailbox($vars));
+            $view_ob->checkToken($vars);
             return $view_ob->downloadAll();
 
         case 'download_attach':
             $view_ob = new IMP_Contents_View(new IMP_Indices_Mailbox($vars));
+            $view_ob->checkToken($vars);
             return $view_ob->downloadAttach($vars->id, $vars->zip);
 
         case 'download_mbox':
@@ -556,6 +562,7 @@ class IMP_Application extends Horde_Registry_Application
 
         case 'download_render':
             $view_ob = new IMP_Contents_View(new IMP_Indices_Mailbox($vars));
+            $view_ob->checkToken($vars);
             return $view_ob->downloadRender($vars->id, $vars->mode, $vars->ctype);
 
         case 'save_message':
@@ -566,4 +573,17 @@ class IMP_Application extends Horde_Registry_Application
         return array();
     }
 
+    /* Exception handler. */
+
+    /**
+     */
+    public function exceptionHandler(Exception $e)
+    {
+        if ($e instanceof Horde_Imap_Client_Exception) {
+            $e = new Horde_Exception_AuthenticationFailure($e->getMessage(), Horde_Auth::REASON_MESSAGE);
+        }
+
+        Horde_ErrorHandler::fatal($e);
+    }
+
 }
