@@ -186,7 +186,7 @@ class IMP_Basic_Compose extends IMP_Basic_Base
                 if (isset($_FILES['upload_' . $i]) &&
                     strlen($_FILES['upload_' . $i]['name'])) {
                     try {
-                        $atc_ob = $imp_compose->addAttachmentFromUpload($this->vars, 'upload_' . $i);
+                        $atc_ob = $imp_compose->addAttachmentFromUpload('upload_' . $i);
                         if ($notify) {
                             $notification->push(sprintf(_("Added \"%s\" as an attachment."), $atc_ob->getPart()->getName()), 'horde.success');
                         }
@@ -389,7 +389,7 @@ class IMP_Basic_Compose extends IMP_Basic_Base
             try {
                 $imp_compose->redirectMessage($this->indices);
                 $redirect = true;
-                $this->title = ngettext(_("Redirect"), _("Redirect Messages"), count($this->indices));
+                $this->title = ngettext("Redirect", "Redirect Messages", count($this->indices));
             } catch (IMP_Compose_Exception $e) {
                 $notification->push($e, 'horde.error');
             }
