@@ -699,7 +699,7 @@ class IMP_Basic_Message extends IMP_Basic_Base
 
         $imp_params = $mailbox->urlParams($buid);
         $a_view->save_as = Horde::widget(array(
-            'url' => $registry->downloadUrl($subject, array_merge(array('actionID' => 'save_message'), $imp_params)),
+            'url' => IMP_Contents_View::downloadUrl($subject, array_merge(array('actionID' => 'save_message'), $imp_params)),
             'title' => _("Sa_ve as"),
             'nocheck' => true
         ));
@@ -1019,7 +1019,7 @@ class IMP_Basic_Message extends IMP_Basic_Base
                     $output .= '&nbsp;(' . $val2 . ')';
                 }
                 break;
-            } elseif ($url = $text_filter->filter($val, 'linkurls')) {
+            } elseif ($url = $text_filter->filter($val->url, 'linkurls')) {
                 $output = $url;
                 foreach ($val->comments as $val2) {
                     $output .= '&nbsp;(' . $val2 . ')';
@@ -1028,7 +1028,9 @@ class IMP_Basic_Message extends IMP_Basic_Base
             }
         }
 
-        return $output;
+        return strlen($output)
+            ? $output
+            : htmlspecialchars($data);
     }
 
 }
