@@ -354,6 +354,7 @@ class IMP_Basic_Search extends IMP_Basic_Base
         $view = new Horde_View(array(
             'templatePath' => IMP_TEMPLATES . '/search'
         ));
+        $view->addHelper('Text');
         $view->addHelper('Tag');
         $view->addHelper('FormTag');
 
@@ -464,8 +465,10 @@ class IMP_Basic_Search extends IMP_Basic_Base
         }
 
         /* Prepare the topbar. */
-        $injector->getInstance('Horde_View_Topbar')->subinfo =
-            $injector->getInstance('IMP_View_Subinfo')->render();
+        if (!$dynamic_view) {
+            $injector->getInstance('Horde_View_Topbar')->subinfo =
+                $injector->getInstance('IMP_View_Subinfo')->render();
+        }
 
         Horde_Core_Ui_JsCalendar::init();
         $page_output->addScriptFile('hordecore.js', 'horde');
