@@ -22,6 +22,13 @@
  */
 class IMP_Images
 {
+    /**
+     * Always show inline images?
+     *
+     * @var boolean
+     */
+    public $alwaysShow = false;
+
     /**
      * Show inline images in messages?
      *
@@ -34,7 +41,7 @@ class IMP_Images
     {
         global $injector, $prefs, $registry;
 
-        if (!$prefs->getValue('image_replacement')) {
+        if ($this->alwaysShow || !$prefs->getValue('image_replacement')) {
             return true;
         }
 
