@@ -103,8 +103,12 @@ class IMP_Imap implements Serializable
 
         case 'max_compose_recipients':
         case 'max_compose_timelimit':
+            $perm = $GLOBALS['injector']->getInstance('Horde_Perms')->getPermissions('imp:' . str_replace('max_compose', 'max', $key), $GLOBALS['registry']->getAuth());
+            return intval($perm[0]);
+
         case 'max_create_mboxes':
-            return $GLOBALS['injector']->getInstance('Horde_Perms')->getPermissions('imp:' . $this->_getPerm($key), $GLOBALS['registry']->getAuth());
+            $perm = $GLOBALS['injector']->getInstance('Horde_Perms')->getPermissions('imp:' . $this->_getPerm($key), $GLOBALS['registry']->getAuth());
+            return intval($perm[0]);
 
         case 'server_key':
             return $this->init
@@ -142,7 +146,7 @@ class IMP_Imap implements Serializable
      */
     private function _getPerm($perm)
     {
-        return ($this->init ? $this->server_key . ':' : '') . $perm;
+        return 'backends:' . ($this->init ? $this->server_key . ':' : '') . $perm;
     }
 
     /**
