@@ -242,6 +242,7 @@ class IMP_Imap_Tree implements ArrayAccess, Countable, Iterator, Serializable
          * POP3. */
         if (!$access_folders) {
             $this->_insertElt($this->_makeElt('INBOX', self::ELT_IS_SUBSCRIBED));
+            $this->track = $old_track;
             return;
         }
 
@@ -1401,7 +1402,7 @@ class IMP_Imap_Tree implements ArrayAccess, Countable, Iterator, Serializable
     {
         $str = strval($name);
 
-        return ((strlen($str) == 5) && (strcasecmp($str, 'INBOX') == 0))
+        return ((strlen($str) == 5) && (strcasecmp($str, 'INBOX') === 0))
             ? 'INBOX'
             : $str;
     }
